/*
   $Id: applet_proxy.cpp,v 1.3 2006/07/14 09:22:38 noir Exp $
  Copyright (C) 2006 Hiroyuki KUROSAKI <noir (at) st . rim . or . jp>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "version.h"
#include "applet_proxy.h"
#include <qpainter.h>
#include <qpe/resource.h>
#include <qpe/config.h>
#include <qmessagebox.h>

#include "rb_funcall2_protect.h"

using namespace RubyQpeApplet;

AppletProxy::AppletProxy(QWidget* parent)
    : QWidget(parent)
{
    init_applet();

    init_taskbar_icon();

    init_qcop_channel();
}

AppletProxy::~AppletProxy()
{
    if (!NIL_P(applet)) {
        finalize_applet();
    }
}

void AppletProxy::paintEvent(QPaintEvent*)
{
    QPainter painter(this);

    painter.drawPixmap(0,
                       (int)((height() - taskbar_icon.height()) / 2),
                       taskbar_icon);
}

void AppletProxy::mousePressEvent(QMouseEvent*)
{
    if (NIL_P(applet)) {
        QMessageBox::information(this, "No Applet", "No Applet loaded.");
    }
    else {
        VALUE signal_clicked = rb_funcall2_protect(applet,
                                                   rb_intern("clicked"),
                                                   0, 0, &ruby_state);
        if (!NIL_P(signal_clicked)) {
            rb_funcall2_protect(signal_clicked,
                                rb_intern("send"), 0, 0, &ruby_state);
            if (ruby_state) {
                VALUE args[1];
                args[0] = ruby_errinfo;
                rb_funcall2_protect(rb_cObject,
                                    rb_intern("log_error"),
                                    1, args, &ruby_state);
                applet = Qnil;
            }
        }
        else {
            QMessageBox::information(this, "Applet", "Signal clicked not found.");
        }
    }
}

void AppletProxy::init_applet()
{
    applet = Qnil;
    ruby_state = 0;

    Config config(APP_NAME);
    config.setGroup("Applet");
    QString file_name(config.readEntry("file_name"));
    QString class_name(config.readEntry("class_name"));
    config.setGroup("Log");
    QString  log_file_name(config.readEntry("file_name"));

    ruby_init();
    ruby_init_loadpath();

    char library_name[] = "qpeapplet";

    ruby_script(library_name);

    rb_protect((VALUE (*)(VALUE))rb_require,
               (VALUE)library_name,
               &ruby_state);

    if (ruby_state) {
// log_error not available yet...
    }
    else {
        if (!file_name.isNull()) {
            rb_protect((VALUE (*)(VALUE))rb_require,
                       (VALUE)(const char*)file_name.utf8(),
                       &ruby_state);
        }
        if (file_name.isNull() || class_name.isNull()) {
            class_name = "Qpe::Applet::NotFoundApplet";
        }

        VALUE args[1];
        args[0] = rb_str_new2("Qpe");

        VALUE mQpe = rb_funcall2_protect(rb_cObject,
                                         rb_intern("const_get"),
                                         1, args, &ruby_state);

        VALUE mApplet = Qnil;
        args[0] = rb_str_new2("Applet");
        if (!ruby_state && !NIL_P(mQpe)) {
            mApplet = rb_funcall2_protect(mQpe,
                                          rb_intern("const_get"),
                                          1, args, &ruby_state);
        }

        args[0] = rb_str_new2((const char*)log_file_name.utf8());
        if (!ruby_state && !NIL_P(mApplet)) {
            rb_funcall2_protect(mApplet,
                                rb_intern("log_file="),
                                1, args, &ruby_state);
        }

        QString code(class_name + ".new");

        applet = rb_eval_string_protect((const char*)code.utf8(),
                                        &ruby_state);
        if (ruby_state) {
            args[0] = ruby_errinfo;
            rb_funcall2_protect(rb_cObject,
                                rb_intern("log_error"),
                                1, args, &ruby_state);
            applet = Qnil;
        }
    }
}

void AppletProxy::finalize_applet()
{
    if (!NIL_P(applet)) {
        applet = Qnil;
        ruby_finalize();
    }
}

void AppletProxy::reload()
{
    finalize_applet();
    init_taskbar_icon();
    update();
    init_applet();
}

void AppletProxy::init_taskbar_icon()
{
    Config config(APP_NAME);

    config.setGroup("Applet");
    QString name(config.readEntry("taskbar_icon", APP_NAME));

    taskbar_icon = Resource::loadPixmap(name);
    setFixedWidth(taskbar_icon.width());
}

void AppletProxy::init_qcop_channel()
{
    qcop_channel = new QCopChannel("QPE/RubyQpeApplet", this);
    connect(qcop_channel,
            SIGNAL(received(const QCString&, const QByteArray&)),
            this,
            SLOT(receive_message(const QCString&, const QByteArray&)));
}

void AppletProxy::receive_message(
    const QCString& msg, const QByteArray&)
{
    if (msg == "reload()") {
        reload();
    }
}
