# $Id: qpeapplet.rb,v 1.5 2006/07/14 09:22:38 noir Exp $

require 'qte'
require 'qpe'
require 'qtedecl'

include Qte

module Qpe
  module Applet

    @@log_file = '/dev/null'

    def self.log_file=(new_file)
      @@log_file = new_file
    end

    def self.log_file
      @@log_file
    end

    def self.log_error(errinfo)
      begin
        File.open(@@log_file, 'a') do |f|
          f.puts '---'
          f.puts Time.now
          f.puts errinfo.class.name
          f.puts errinfo.message
          errinfo.backtrace.each do |bt|
            f.puts "\tfrom " + bt
          end
        end
      rescue
        nil
      end
    end

    class AppletError <RuntimeError; end
    # Base class of Ruby Qpe Applet.
    # User must define subclass of this.
    class Base <QWidget
      def initialize(parent = nil, name = '', wflags = 0)
        super
        begin
          user_interface
          @clicked = RSignal.new
          connect(@clicked, self, "toggle_show_hide")
        rescue
          Qpe::Applet.log_error($!)
        end
      end

      attr_reader :clicked

      # Define user interface (or anything) in this method.
      def user_interface
        nil
      end

      # default slot for the clicked signal.
      def toggle_show_hide
        begin
          if self.isVisible then
            self.hide
          else
            self.raise
            self.show
          end
        rescue
          Qpe::Applet.log_error($!)
        end
      end
    end

    class NotFoundApplet <Base
      def user_interface
        begin
          self.caption = 'Applet Not Found'
          lb = label('Applet not found.')
          self.fixedSize = lb.sizeHint
        rescue
          Qpe::Applet.log_error($!)
        end
      end
    end

  end
end

def log_error(errinfo)
  Qpe::Applet.log_error(errinfo)
end
